#include "beepblox.h"

//Library Loads and plays a raw WAV file from the SD card
//File must have sampling rate of 31.25kHz for playback speed to be correct
//Card must not be fragmented (full format required if any files are deleted)
BeepBloX::BeepBloX() //: SdPlayClass()
{
}
	
bool BeepBloX::init(void) 
{
	// If your SD card CS-Pin is not at Pin 4, enable and adapt the following line:
	SdPlay.setSDCSPin(CS_PIN);
	initialized=true;
	
	#ifdef BeepBloXDebug
		{
			// Open serial communications 
			Serial.begin(9600);
			
			Serial.print(F("Free Ram: "));
			Serial.println(freeRam());
			
			// Using F("...") to avoid wasting RAM  
			Serial.println(F("\nInitializing SD card..."));  
			
			// Select between SSDA_MODE_FULLRATE or SSDA_MODE_HALFRATE (62.5kHz or 31.25kHz)
			// and the output modes SSDA_MODE_MONO_BRIDGE, SSDA_MODE_MONO or SSDA_MODE_STEREO
			if (!SdPlay.init(SSDA_MODE_HALFRATE | SSDA_MODE_MONO | SSDA_MODE_AUTOWORKER)) {
				Serial.println(F("!Initialization failed! \nThings to check:"));
				Serial.println(F("* Is a card is inserted?"));
				Serial.println(F("* Is your wiring correct?" ));
				Serial.print(F("\tCS=" ));	Serial.println(CS_PIN);
				Serial.println(F("\tSCK=D13\n\tMOSI=11\n\tMISO=12\n\tVCC=V\n\tGND=G" ));
				Serial.print(F("Error code: "));
				Serial.println(SdPlay.getLastError());
				while(1);
				} else {
				Serial.println(F("Wiring is correct and a card is present.")); 
			}
		}
		#else
		{
			return SdPlay.init(SSDA_MODE_HALFRATE | SSDA_MODE_MONO | SSDA_MODE_AUTOWORKER);
		}
	#endif

}

//"Loads" a raw WAV file from the SD card
//File must have sampling rate of 31.25kHz for playback speed to be correct PCM 8bit
//Card must not be fragmented (full format required if any files are deleted)
//Files must be 8.3 filename length
bool BeepBloX::setFile(String AudioFileName)
{
  if(!initialized)
    init();

  AudioFileName.toUpperCase();
  if(AudioFileName.indexOf(".WAV")==-1)  //Check for .WAV extension 
    AudioFileName+=".WAV";

  //const char* filename = AudioFileName.c_str();
    
  #ifdef BeepBloXDebug
    {     
      Serial.println(F("\nList of files on card:"));
      SdPlay.dir(&BeepBloX::DirCallback);

      Serial.print(F("\nLooking for: "));
      Serial.println(AudioFileName);
      if(!SdPlay.setFile(AudioFileName.c_str())) {
        Serial.println(F("File not found on card! Error code: "));
        Serial.println(SdPlay.getLastError());
        return false;
        } else {
        Serial.println(F("File found.")); 
        return true;
      }   
    }
    #else
    {
      return SdPlay.setFile(AudioFileName.c_str());
    }
  #endif
}

//Begins playback of the audio file 
//Runs in the background via two interrupts 
//(one for setting the raw PWM pin, and the other to load a chunk of the file from the SD card into the buffer)
//waitdone: false = run in the background, code will continue
//true = function will not exit until playback is complete
bool BeepBloX::play(bool waitdone /*=false*/)
{
	SdPlay.play();
	if(waitdone) 
		while(!SdPlay.isStopped())
			{}
}

//Lists all files found on the SD card
//Returned via the serial port
void BeepBloX::listFiles()
{
  //if(!initialized)
    //init();
  
  //setFile("none"); //Required to init the card before listing
  //SdPlay.play();
    
  Serial.println(F("\n\nList of files on card:"));
  delay(15);  //Help with mBlock time outs mid filename
  SdPlay.dir(&BeepBloX::DirCallback);
  Serial.println(F("------------\n\n"));
}


	// Callback target, prints output to serial
	void BeepBloX::DirCallback(char *buf) { 
		Serial.println(buf);
    delay(15);  //Help with mBlock time outs mid filename
    //Serial.println("       \xff\x55"); //Keep mBlock happy
	}

#ifdef BeepBloXDebug
	// helper function to determine free ram at runtime
	int BeepBloX::freeRam () {
		extern int __heap_start, *__brkval; 
		int v; 
		return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
	}
#endif
