#include "faceblox.h"

//FaceBloX::FaceBloX() : MD_MAX72XX(DATA_PIN, CLK_PIN, CS_PIN, MAX_DEVICES)
FaceBloX::FaceBloX() : MD_MAX72XX(CS_PIN, MAX_DEVICES)
{
	begin();
}

FaceBloX::FaceBloX(uint8_t numDisplays) : MD_MAX72XX(CS_PIN, numDisplays) 
{
	begin();
}

//    FaceBloX():
//      _dataPin(DATA_PIN), _clkPin(CLK_PIN), _csPin(CS_PIN), _maxDevices(2),
//      _updateEnabled(true), _hardwareSPI(false)
//    {
//    }
//
//    FaceBloX(uint8_t dataPin, uint8_t clkPin, uint8_t csPin, uint8_t numDevices):
//      _dataPin(dataPin), _clkPin(clkPin), _csPin(csPin), _maxDevices(numDevices),
//      _updateEnabled(true), _hardwareSPI(false)
//    {
//    }
//
//    FaceBloX(uint8_t csPin, uint8_t numDevices):
//      _dataPin(0), _clkPin(0), _csPin(csPin), _maxDevices(numDevices),
//      _updateEnabled(true), _hardwareSPI(true)
//    {
//    }

//Load a bitmap (8x16) image onto the display
//Each byte represents a column (a0=left most, h1=right most)
//If only one display is connected then only the right half of the image will be displayed (a1-h1)
void FaceBloX::picture(byte a0, byte b0, byte c0, byte d0, byte e0, byte f0, byte g0, byte h0,
byte a1, byte b1, byte c1, byte d1, byte e1, byte f1, byte g1, byte h1)
{
	static byte bitmap[17];
	
	bitmap[0] = a0;       bitmap[1] = b0;      bitmap[2] = c0;      bitmap[3] = d0;      bitmap[4] = e0;      bitmap[5] = f0;      bitmap[6] = g0;      bitmap[7] = h0;     //Move the first 8 bytes into the buffer
	bitmap[0 + 8] = a1; bitmap[1 + 8] = b1; bitmap[2 + 8] = c1; bitmap[3 + 8] = d1; bitmap[4 + 8] = e1; bitmap[5 + 8] = f1; bitmap[6 + 8] = g1; bitmap[7 + 8] = h1; //Move the second set of 8 bytes into the buffer
	
	update(MD_MAX72XX::OFF);      //Disable updates so we can load and flip the buffer before displaying
	setBuffer(15, 16, bitmap );  //start at column 15, draw 16 columns - if only using 1 Matrix the 2nd half (9-16) will be displayed
	transform(MD_MAX72XX::TFUD);  //Invert vertically - The matrix from mBlock is up-side-down (reverse bit order)
	update(MD_MAX72XX::ON);       //Update display with new image
}

//Load a bitmap (8x16) image onto the display (16 byte array)
//Each byte represents a column (0=left most, 15=right most)
//If only one display is connected then only the right half of the image will be displayed (8-15)
void FaceBloX::picture(byte* bitmap)
{
	update(MD_MAX72XX::OFF);      //Disable updates so we can load and flip the buffer before displaying
	setBuffer(15, 16, bitmap );   //start at column 15, draw 16 columns - if only using 1 Matrix the 2nd half (9-16) will be displayed
	transform(MD_MAX72XX::TFUD);  //Invert vertically - The matrix from mBlock is up-side-down (reverse bit order)
	update(MD_MAX72XX::ON);       //Update display with new image
}

//Overload to handle a blank picture
//Clears the display
void FaceBloX::picture()
{
	clear();      //Clear all displays
}


//Scrolls a message synchronously (the whole message will be displayed before returning)
//scrollSpeed: Speed to scroll as a percentage. 0=no movement (beginning of message will be loaded, then return)
//charSpacing: number of blank columns between characters
bool FaceBloX::scrollText(const char *message, uint8_t scrollSpeed, uint8_t charSpacing)
{
	if(scrollSpeed >  100 || scrollSpeed <=  0)
	{
		showText(message, charSpacing);
		return true;
	}
	
	if(charSpacing >  20) charSpacing = 20;
	if(charSpacing <  0) charSpacing = 0;
	
	double dly = (100 - scrollSpeed) * 0.4 + 1.0;	//Scale exponentially to give finer control at the higher speeds
	uint32_t scrollDelay = (uint32_t)(dly*dly);
	while (scrollTextASync(message, scrollDelay, charSpacing))
		;
}


//Scrolls a message synchronously (the whole message will be displayed before returning)
//scrollSpeed: ms between scrolling a column. 0=no movement (beginning of message will be loaded, then return)
//charSpacing: number of blank columns between characters
bool FaceBloX::scrollText(String message, uint8_t scrollSpeed, uint8_t charSpacing)
{
	scrollText(message.c_str(), scrollSpeed, charSpacing);
}


//Scrolls a message synchronously (the whole message will be displayed before returning)
//scrollSpeed: ms between scrolling a column. 0=no movement (beginning of message will be loaded, then return)
//charSpacing: number of blank columns between characters
bool FaceBloX::scrollText(double message, uint8_t scrollSpeed, uint8_t charSpacing)
{
	if((int)message == message)
		scrollText(String((int)message).c_str(), scrollSpeed, charSpacing);
	else
		scrollText(String(message).c_str(), scrollSpeed, charSpacing);
}

//Scrolls a message synchronously (the whole message will be displayed before returning)
//scrollSpeed: ms between scrolling a column. 0=no movement (beginning of message will be loaded, then return)
//charSpacing: number of blank columns between characters
// bool FaceBloX::scrollText(long int message, uint8_t scollDelay, uint8_t charSpacing)
// {
	// scrollText(String(message).c_str(), scollDelay, charSpacing);
// }



//Scrolls a message asynchronously (this method must be called repeatedly)
//scollDelay: ms between scrolling a column. 0=no movement
//charSpacing: number of blank columns between characters
bool FaceBloX::scrollTextASync(const char *message, uint32_t scollDelay, uint8_t charSpacing)
{
	static bool bInit = true;
	static const char   *p;
	static uint8_t  state = 0;
	static uint8_t  curLen, showLen;
	static uint8_t  cBuf[8];
	uint8_t         colData;
	static uint32_t prevTimeAnim = 0;    // Used for remembering the millis() value in animations
	
	
	// Are we initializing?
	if (bInit)
	{
        state = 0;
        p = message;
        prevTimeAnim = 0;      // starting point for next time
        transform(MD_MAX72XX::TSL);  // scroll along 1 extra column to push anything on the display over (looks better)
        bInit = false;
	}
	
	// Is it time to scroll the text?
	if (millis() - prevTimeAnim < scollDelay)
	return (1);
	
	// scroll the display
	transform(MD_MAX72XX::TSL);  // scroll along
	prevTimeAnim = millis();      // starting point for next time
	
	// now run the finite state machine to control what we do
	switch (state)
	{
        case 0: // Load the next character from the font table
		showLen = getChar(*p++, sizeof(cBuf) / sizeof(cBuf[0]), cBuf);
		curLen = 0;
		state = 1;
		
        // !! deliberately fall through to next state to start displaying
		
        case 1: // display the next part of the character
		colData = cBuf[curLen++];
		setColumn(0, colData);
		if (curLen == showLen)
		{
            showLen = ((*p != '\0') ? charSpacing : getColumnCount() - 1);
            curLen = 0;
            state = 2;
		}
		break;
		
        case 2: // display inter-character spacing (blank column) or scroll off the display
		setColumn(0, 0);
		if (++curLen == showLen)
		{
            state = 0;
            if (*p == '\0') //If we reached the end of the string
            {
				transform(MD_MAX72XX::TSL);  // scroll final column off
				bInit = true;
				return 0;
			}
            return 1;
		}
		break;
		
        default:
		state = 0;
	}
	
	return (1);
}


//Shows a message (If the message is larger than the display only the first part will be displayed)
//charSpacing: number of blank columns between characters
bool FaceBloX::showText(const char *message, uint8_t charSpacing)
{
	int i=0;
	clear();      //Clear all displays
	for(int col=getColumnCount()-1;col>=0 && message[i]!='\0';col-=charSpacing)
	{
		col -= setChar(col, message[i++]);	//Place a character (and get the width it took)
	}
}
