
#include "firmware_roboblox.h"

namespace Firmware
{
String mVersion = "\xac\xe2\xc0\xde\x21\x11\x11\x11\x11";     //ace2code21 = RoboBloX 2.1

//Servo servos[14];

/*
  typedef struct MeModule
  {
    int device;
    int port;
    int slot;
    int pin;
    int index;
    float values[3];
  } MeModule;
*/
union {
  byte byteVal[4];
  float floatVal;
  long longVal;
} val;

union {
  byte byteVal[8];
  double doubleVal;
} valDouble;

union {
  byte byteVal[2];
  short shortVal;
} valShort;

//MeModule modules[12];
int analogs[8] = {A0, A1, A2, A3, A4, A5, A6, A7};
boolean isAvailable = false;

extern int len = 52;
char buffer[52];
char bufferBt[52];
char resultbuf[52];
byte resultbuflen = 0;
byte index = 0;
byte dataLen;
byte modulesLen = 0;
boolean isStart = false;
unsigned char irRead;
char serialRead;


float angleServo = 90.0;
unsigned char prevc = 0;
double lastTime = 0.0;
double currentTime = 0.0;
uint8_t keyPressed = 0;
uint8_t command_index = 0;

char _receiveStr[20] = {};
uint8_t _receiveUint8[16] = {};

FaceBloX faceblox;



void RoboBloX_mBlock_Live() {
  Serial.begin(115200);
  Serial.println(mVersion);

  while (1)   //It is not easy to place a function call to this code in the loop due to mBlock, thus using a while(1) instead
  {
    readSerial();
    if (isAvailable) {
      unsigned char c = serialRead & 0xff;
      if (c == 0x55 && isStart == false) {
        if (prevc == 0xff) {
          index = 1;
          isStart = true;
        }
      } else {
        prevc = c;
        if (isStart) {
          if (index == 2) {
            dataLen = c;
          } else if (index > 2) {
            dataLen--;
          }
          writeBuffer(index, c);
        }
      }
      index++;
      if (index > 51) {
        index = 0;
        isStart = false;
      }
      if (isStart && dataLen == 0 && index > 3) {
        isStart = false;
        parseData();
        index = 0;
      }
    }
  }
}
unsigned char readBuffer(int index) {
  return buffer[index];
}
void writeBuffer(int index, unsigned char c) {
  buffer[index] = c;
}
void writeHead() {
  writeSerial(0xff);
  writeSerial(0x55);
}
void writeEnd() {
  Serial.println();
}
void writeResult() {
  Serial.write(resultbuf, resultbuflen);
  resultbuflen=0; //Empty buffer
}
void writeSerial(unsigned char c) {
  Serial.write(c);
}

void writeSerialBuffer(unsigned char c) {
  resultbuf[resultbuflen++]=c;;
}

void readSerial() {
  isAvailable = false;
  if (Serial.available() > 0) {
    isAvailable = true;
    serialRead = Serial.read();
  }
}
/*
  ff 55 len idx action device port  slot  data a
  0  1  2   3   4      5      6     7     8
*/
void parseData() {
  isStart = false;
  int idx = readBuffer(3);
  command_index = (uint8_t)idx;
  int action = readBuffer(4);
  int device = readBuffer(5);
  switch (action) {
    case GET: {
        readSensor(device); //Read sensor (store result in result buffer)
        
        writeHead();
        writeSerial(idx);
        writeResult();
        writeEnd();
      }
      break;
    case RUN: {
        runModule(device);
        callOK();
      }
      break;
    case RESET: {
        callOK();
      }
      break;
    case START: {
        callOK();
      }
      break;
  }
}
void callOK() {
  writeSerial(0xff);
  writeSerial(0x55);
  writeEnd();
}
void sendByte(char c) {
  writeSerialBuffer(1);
  writeSerialBuffer(c);
}
void sendString(String s) {
  int l = s.length();
  writeSerialBuffer(4);
  writeSerialBuffer(l);
  for (int i = 0; i < l; i++) {
    writeSerialBuffer(s.charAt(i));
  }
}
void sendFloat(float value) {
  writeSerialBuffer(0x2);
  val.floatVal = value;
  writeSerialBuffer(val.byteVal[0]);
  writeSerialBuffer(val.byteVal[1]);
  writeSerialBuffer(val.byteVal[2]);
  writeSerialBuffer(val.byteVal[3]);
}
void sendShort(double value) {
  writeSerialBuffer(3);
  valShort.shortVal = value;
  writeSerialBuffer(valShort.byteVal[0]);
  writeSerialBuffer(valShort.byteVal[1]);
}
void sendDouble(double value) {
  writeSerialBuffer(2);
  valDouble.doubleVal = value;
  writeSerialBuffer(valDouble.byteVal[0]);
  writeSerialBuffer(valDouble.byteVal[1]);
  writeSerialBuffer(valDouble.byteVal[2]);
  writeSerialBuffer(valDouble.byteVal[3]);
}
short readShort(int idx) {
  valShort.byteVal[0] = readBuffer(idx);
  valShort.byteVal[1] = readBuffer(idx + 1);
  return valShort.shortVal;
}
float readFloat(int idx) {
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx + 1);
  val.byteVal[2] = readBuffer(idx + 2);
  val.byteVal[3] = readBuffer(idx + 3);
  return val.floatVal;
}
long readLong(int idx) {
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx + 1);
  val.byteVal[2] = readBuffer(idx + 2);
  val.byteVal[3] = readBuffer(idx + 3);
  return val.longVal;
}
char* readString(int idx, int len) {
  for (int i = 0; i < len; i++) {
    _receiveStr[i] = readBuffer(idx + i);
  }
  _receiveStr[len] = '\0';
  return _receiveStr;
}
uint8_t* readUint8(int idx, int len) {
  for (int i = 0; i < len; i++) {
    if (i > 15) {
      break;
    }
    _receiveUint8[i] = readBuffer(idx + i);
  }
  return _receiveUint8;
}

//==============================================================================================================================
void runModule(int device) {
  //0xff 0x55 0x6 0x0 0x1 0xa 0x9 0x0 0x0 0xa
  /**************************************************
    ff 55 len idx action device pin slot data a
    0  1  2   3   4      5      6    7    8
  ***************************************************/
  int pin = readBuffer(6);
  //int pin = port;

  switch (device) {
    case DIGITAL: {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        digitalWrite(pin, v);
      }
      break;

    case PWM: {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        analogWrite(pin, v);
      }
      break;

    case TONE: {
        pinMode(pin, OUTPUT);
        int hz = readShort(7);
        int ms = readShort(9);
        if (ms > 0) {
          tone(pin, hz, ms);
        } else {
          noTone(pin);
        }
      }
      break;

//    case SERVO: {
//        int angle = readBuffer(7);
//        Servo servo = servos[pin];
//        if(angle<0)
//          angle=0;
//        if(angle>180)
//          angle=180;
// 
//        if(!servo.attached())
//         servo.attach(pin);
// 
//        if(servo.read() != angle)
//          servo.write(angle);
//         
//      }
//      break;


    case TIMER: {
        lastTime = millis() / 1000.0;
      }
      break;

    //------------ROBOBLOX-------------
    case BLINKBLOX: {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        digitalWrite(pin, v);
      }
      break;

    case RAINBOWBLOX:
    case RGBLED:
      {
        signed char color = (signed char)readBuffer(7);
        int pinRd = (int)readBuffer(8);
        int pinYw = (int)readBuffer(9);
        int pinGn = (int)readBuffer(10);
        //led.reset(port,slot);

      }
      break;


    case TRAFFICBLOX: {
        signed char color = (signed char)readBuffer(7);
        int pinRd = (int)readBuffer(8);
        int pinYw = (int)readBuffer(9);
        int pinGn = (int)readBuffer(10);
        RoboBloX::TrafficBloX(color, pinRd, pinYw, pinGn);
      }
      break;


    case BEEPBLOX: {
        int action = readBuffer(7);
        switch (action) {
          case 1: { //Tone
              int hz = readShort(8);
              int ms = readShort(10);
              pinMode(pin, OUTPUT);
              if (ms > 0) {
                tone(pin, hz, ms);
              } else {
                noTone(pin);
              }
            }
            break;
          case 2: { //Wav
              static BeepBloX beepblox;
              bool wait = (bool)readBuffer(8);
              int len = readBuffer(9);
              //const char *filename = readString(10, len);
              String filename = String(readString(10, len));
              filename.toUpperCase();
              
              if( filename=="LS" || filename=="LIST" )
                {
                  //callOK(); //Return function complete signal before listing files (to avoid mBlock time out)
                  beepblox.listFiles();
                }
              else
                {
                  beepblox.setFile(filename);
                  beepblox.play(wait);
                }
            }
            break;
          case 3: { //List files
              static BeepBloX beepblox;
              beepblox.listFiles();
            }
            break;
        }
      }
      break;

    case GOBLOX: {
        static GoBloX goblox;
        int action = readBuffer(6);
        switch (action) {
          case 1: { //Drive
              char dir = readBuffer(7);
              signed char speed = (signed char)readBuffer(8);
              goblox.drive(dir, speed);
              break;
            }

          case 2: { //DriveDual
              signed char left = (signed char)readBuffer(7);
              signed char right = (signed char)readBuffer(8);
              goblox.driveDual(left, right);
              break;
            }
        }
      }
      break;


    case FACEBLOX:
      //case LEDMATRIX:
      {
        int action = readBuffer(7);
        switch (action) {
          case 0: { //Reset
              faceblox.begin();
              break;
            }
          case 1: { //scrollText
              unsigned int scrollDelay = buffer[8];
              unsigned int charSpacing = buffer[9];
              int len = readBuffer(10);
              const char *msg = readString(11, len);
              faceblox.scrollText(msg, scrollDelay, charSpacing);
              break;
            }
          case 2: { //picture
              //int px = readBuffer(8);
              //int py = readBuffer(9);
              uint8_t *pic = readUint8(10, 16);
              faceblox.picture(pic);
              break;
            }
          case 3: { //showClock
              int point = readBuffer(8);
              int hours = readBuffer(9);
              int minutes = readBuffer(10);
              //   ledMx.showClock(hours,minutes,point);
              break;
            }
        }
      }
      break;





    case SEVSEG: {
        // if(seg.getPort()!=port){
        // seg.reset(port);
        // }
        // float v = readFloat(7);
        // seg.display(v);
      }
      break;




  }
}

//==============================================================================================================================
void readSensor(int device) {
  /**************************************************
      ff 55 len idx action device pin slot data a
      0  1  2   3   4      5      6    7    8
  ***************************************************/
  float value = 0.0;
  signed char pin = readBuffer(6);

  switch (device) {
    case  VERSION: {
        sendString(mVersion);
      }
      break;

    case  DIGITAL: {
        if(pin >= 0)
            {
              pinMode(pin, INPUT); 
              sendByte(digitalRead(pin));
            }
          else
            {
              pinMode(-pin, INPUT_PULLUP); //Negative pin number will enable pullup
              sendByte(digitalRead(-pin));
            }
      }
      break;

    case  ANALOG: {
        //pin = analogs[pin];
        pinMode(A0 + pin, INPUT);
        sendShort(analogRead(pin));
      }
      break;

    case  PULSEIN: {
        int pw = readShort(7);
        pinMode(pin, INPUT);
        sendShort(pulseIn(pin, HIGH, pw));
      }
      break;

    case ULTRASONIC:
    case BATBLOX:
      {
        int trig = readBuffer(6);
        int echo = readBuffer(7);
        int dist = RoboBloX::BatBloX(trig, echo);
        sendFloat(dist);
      }
      break;

    case TIMER: {
        sendFloat((float)currentTime);
      }
      break;

    //----------RoboBloX---------------
    case  BUTTONBLOX: {
        pinMode(pin, INPUT_PULLUP);
        sendByte(!digitalRead(pin));
      }
      break;

    case  LINEBLOX: {
        //pin = analogs[pin];
        pinMode(A0 + pin, INPUT);
        sendShort(analogRead(pin));
      }
      break;

  }
}

}
